@@ -21,13 +21,13 @@
 #include "includefirst.hpp"
 
 //#include <list>
-#ifndef _MSC_VER
-#  include <rpc/rpc.h>
-#  include <algorithm>
-#else
+#if defined(__CYGWIN__) || defined(_WIN32)
 #  include <rpc/xdr.h>
+#else
+#  include <rpc/rpc.h>
 #endif
 
+#include <algorithm>
 #include "dimension.hpp"
 #include "gdlexception.hpp"
 
@@ -563,7 +563,9 @@ public:
 
   // library functions
   virtual BaseGDL* Convol( BaseGDL* kIn, BaseGDL* scaleIn, BaseGDL* bias,
- 			   bool center, bool normalize, int edgeMode);
+ 			   bool center, bool normalize, int edgeMode,
+                                bool doNan, BaseGDL* missing, bool doMissing,
+                                BaseGDL* invalid, bool doInvalid);
   virtual BaseGDL* Rebin( const dimension& newDim, bool sample);
   // for STRUCT_ASSIGN
   virtual void Assign( BaseGDL* src, SizeT nEl);
