@@ -1,9 +1,15 @@
 /***************************************************************************
+
                           basic_fun.cpp  -  basic GDL library function
+
                              -------------------
+
     begin                : March 14 2004
+
     copyright            : (C) 2004 by Christopher Lee
+
     email                : leec_gdl@publius.co.uk
+
  ***************************************************************************/
 
 /***************************************************************************
@@ -74,6 +80,36 @@ namespace lib {
 #endif
   }
 
+  BaseGDL* gshhg_exists( EnvT* e )
+  {
+#ifdef USE_GSHHS
+    //    e->Message( "GDL was compiled with support for GSHHG" );
+    return new DIntGDL(1);
+#else
+    //e->Message( "GDL was compiled without support for GSHHG" );
+    return new DIntGDL(0);
+#endif
+  }
+  
+  BaseGDL* proj4_exists( EnvT* e )
+  {
+#if defined(USE_LIBPROJ4)
+    return new DIntGDL(1);
+#else
+    return new DIntGDL(0);
+#endif
+  }
+  
+  BaseGDL* proj4new_exists( EnvT* e )
+  {
+#if defined(USE_LIBPROJ4_NEW)
+    return new DIntGDL(1);
+#else
+    return new DIntGDL(0);
+#endif
+  }
+
+
   double Gregorian2Julian(struct tm *ts)
   {
     double jd;
@@ -521,13 +557,4 @@ namespace lib {
     assert(false);
     return NULL;
   }
-
-
-
-
-      
-
-
-			  			  
- 
 } // namespace
