@@ -20,8 +20,6 @@
 #include <iostream>
 #ifdef _MSC_VER
 #include <io.h> // isatty, windows
-#define feclearexcept(e)
-#define FE_ALL_EXCEPT
 #else
 #include <unistd.h> // isatty
 #endif
@@ -109,7 +107,7 @@ DInterpreter::DInterpreter(): GDLInterpreter()
 // at main level
 RetCode GDLInterpreter::NewInterpreterInstance( SizeT lineOffset)
 {
-  if( callStack.size() <= 1) return RC_ABORT; // stay in main loop 
+  if( callStack.size() <= 1 && lineOffset == 0) return RC_ABORT; // stay in main loop 
   
   assert( dynamic_cast<DInterpreter*>( this) != NULL);
   return static_cast<DInterpreter*>( this)->InnerInterpreterLoop(lineOffset);
@@ -152,8 +150,8 @@ void GDLInterpreter::SetRootL( ProgNodeP tt, DotAccessDescT* aD, BaseGDL* r, Arr
       if( r->IsAssoc())
 	  {
 	      ArrayIndexListGuard guard( aL);
-	      throw GDLException( tt, "File expression not allowed "
-				  "in this context: "+Name(r),true,false);
+	      throw GDLException( tt, "File expression not allowed in this context: "+
+					Name(r),true,false);
 	  }
       DStructGDL* structR=static_cast<DStructGDL*>(r);
       aD->ADRoot(structR, aL); 
@@ -162,9 +160,8 @@ void GDLInterpreter::SetRootL( ProgNodeP tt, DotAccessDescT* aD, BaseGDL* r, Arr
   {
       if( r->Type() != GDL_OBJ)
 	  {
-	      throw GDLException( tt, "Expression must be a"
-				  " STRUCT in this context: "+Name(r),
-				  true,false);
+	      throw GDLException( tt, "Expression must be a STRUCT in this context: "+
+					Name(r),true,false);
 	  }
 
       ArrayIndexListGuard guard( aL);
@@ -233,8 +230,8 @@ if( r->Type() == GDL_STRUCT)
       if( r->IsAssoc())
 	  {
 	      ArrayIndexListGuard guard( aL);
-	      throw GDLException( tt, "File expression not allowed "
-				  "in this context: "+Name(r),true,false);
+	      throw GDLException( tt, "File expression not allowed in this context: "+
+				       Name(r),true,false);
 	  }
       DStructGDL* structR=static_cast<DStructGDL*>(r);
       aD->ADRoot( structR, aL); 
@@ -403,7 +400,7 @@ DStructDesc* GDLInterpreter::GetStruct(const string& name, ProgNodeP cN)
   int proIx=ProIx(proName);
   if( proIx == -1)
     {
-					throw GDLException(cN, "Procedure not found: "+proName, true, false);
+	throw GDLException(cN, "Procedure not found: "+proName, true, false);
     }
   
   // 'guard' call stack
@@ -650,10 +647,12 @@ void AppendExtension( string& argstr)
 DInterpreter::CommandCode DInterpreter::CmdReset()
 {
   RetAll( RetAllException::RESET);
+  return CC_OK;
 }
 DInterpreter::CommandCode DInterpreter::CmdFullReset()
 {
   RetAll( RetAllException::FULL_RESET);
+  return CC_OK;
 }
 
 DInterpreter::CommandCode DInterpreter::CmdCompile( const string& command)
@@ -793,18 +792,19 @@ DInterpreter::CommandCode DInterpreter::ExecuteCommand(const string& command)
     cmdstr = cmdstr.substr(0, sppos);
   }
     
-  //  cout << "Execute command: " << command << endl;
+  //   cout << "Execute command: " << command << endl;
 
   String_abbref_eq cmd( StrUpCase( cmdstr));
 
-  if( cmd( "COMPILE"))
-    {
-      return CmdCompile( command);
-    }
+  // AC: Continue before Compile to have ".c" giving ".continue"
   if( cmd( "CONTINUE"))
     {
       return CC_CONTINUE;
     }
+  if( cmd( "COMPILE"))
+    {
+      return CmdCompile( command);
+    }
   if( cmd( "EDIT"))
     {
       cout << "Can't edit file without running GDLDE." << endl;
@@ -910,14 +910,15 @@ DInterpreter::CommandCode DInterpreter::ExecuteCommand(const string& command)
 void DInterpreter::ExecuteShellCommand(const string& command)
 {
   string commandLine = command;
-  if(commandLine == "") 
-  { 
-    commandLine = GetEnvString("SHELL");
-    if (commandLine == "")
-    {
-      cout << "Error managing child process. Environment variable SHELL not set." << endl;
+  if(commandLine == "") {
+     char* shellEnv = getenv("SHELL");
+	 if (shellEnv == NULL) shellEnv = getenv("COMSPEC");
+	 if (shellEnv == NULL) {
+        cout << "Error managing child process. " <<
+		" Environment variable SHELL or COMSPEC not set." << endl;
       return;
     }
+	 commandLine = shellEnv;
   }
 
   int ignored = system( commandLine.c_str());
@@ -958,7 +959,7 @@ DInterpreter::CommandCode DInterpreter::ExecuteLine( istream* in, SizeT lineOffs
       return ExecuteCommand( line.substr(1));
     }
 
-  // command
+  //  online help (if possible, start a browser)
   if( firstChar == "?") 
     {
       // later, we will have to check whether we have X11/Display or not
@@ -972,6 +973,65 @@ DInterpreter::CommandCode DInterpreter::ExecuteLine( istream* in, SizeT lineOffs
       }
     }
   
+  // shell command
+  if( firstChar == "#") 
+    {
+      if (line.substr(1).length() > 0) {
+	line=line.substr(1);
+	StrTrim(line);
+	line=StrUpCase(line);
+	//cout << "yes ! >>"<<StrUpCase(line)<<"<<" << endl;
+	SizeT nProFun;
+	int nbFound=0;
+	// looking in internal procedures
+	nProFun=libProList.size();
+	for( SizeT i = 0; i<nProFun; ++i)
+	  {
+	    if (line.compare(libProList[ i]->Name()) == 0) {
+	      cout << "Internal PROCEDURE : " << libProList[ i]->ToString() << endl;
+	      nbFound++;
+	      break;
+	    }
+	  }
+	// looking in internal functions
+	nProFun = libFunList.size();
+	for( SizeT i = 0; i<nProFun; ++i)
+	  {
+	    if (line.compare(libFunList[ i]->Name()) == 0) {
+	      cout << "Internal FUNCTION : " << libFunList[ i]->ToString() << endl;
+	      nbFound++;
+	      break;
+	    }
+	  }
+	// looking in compiled functions
+	nProFun = funList.size();
+	for( SizeT i = 0; i<nProFun; ++i)
+	  {
+	    if (line.compare(funList[ i]->Name()) == 0) {
+	      cout << "Compiled FUNCTION : " << funList[ i]->ToString() << endl;
+	      nbFound++;
+	      break;
+	    }
+	  }
+	// looking in compiled procedures
+	nProFun = proList.size();
+	for( SizeT i = 0; i<nProFun; ++i)
+	  {
+	    if (line.compare(proList[ i]->Name()) == 0) {
+	      cout << "Compiled PROCEDURE : " << proList[ i]->ToString() << endl;
+	      nbFound++;
+	      break;
+	    }
+	  }
+	if (nbFound == 0) {
+	  cout << "No Procedure/Function, internal or compiled, with name : "<< line << endl;
+	}
+      } else {
+	cout << "Please provide a pro/fun name !" << endl;
+      }
+      return CC_OK;
+    }
+
   // shell command
   if( firstChar == "$") 
     {
@@ -1317,7 +1377,11 @@ RetCode DInterpreter::InnerInterpreterLoop(SizeT lineOffset)
 {
   ProgNodeP retTreeSave = _retTree;
   for (;;) {
+#if defined (_MSC_VER) && _MSC_VER < 1800
+	_clearfp();
+#else
     feclearexcept(FE_ALL_EXCEPT);
+#endif
 
     DInterpreter::CommandCode ret=ExecuteLine(NULL, lineOffset);
 
@@ -1356,7 +1420,11 @@ bool DInterpreter::RunBatch( istream* in)
 
   while( in->good())
     {
+#if defined (_MSC_VER) && _MSC_VER < 1800
+	  _clearfp();
+#else
       feclearexcept(FE_ALL_EXCEPT);
+#endif
       
       try
 	{
@@ -1399,7 +1467,11 @@ void DInterpreter::ExecuteFile( const string& file)
   bool runCmd = false;
   while( in.good())
     {
+#if defined (_MSC_VER) && _MSC_VER < 1800
+	  _clearfp();
+#else
       feclearexcept(FE_ALL_EXCEPT);
+#endif 
 
       try
  	{
@@ -1481,7 +1553,11 @@ RetCode DInterpreter::InterpreterLoop( const string& startup,
 	{
 	  while( in.good())
 	    {
+#if defined (_MSC_VER) && _MSC_VER < 1800
+		  _clearfp();
+#else
 	      feclearexcept(FE_ALL_EXCEPT);
+#endif
 
 	      try
 		{
@@ -1570,6 +1646,9 @@ RetCode DInterpreter::InterpreterLoop( const string& startup,
   // http://www.delorie.com/gnu/docs/readline/rlman.html
   char rlName[] = "GDL";
   rl_readline_name = rlName;
+  //Our handler takes too long
+  //when editing the command line with ARROW keys. (bug 562). (used also in gdl.cpp)
+  //but... without it we have no graphics event handler! FIXME!!!
   rl_event_hook = GDLEventHandler;
   {
     int edit_input = SysVar::Edit_Input();
@@ -1580,7 +1659,11 @@ RetCode DInterpreter::InterpreterLoop( const string& startup,
   // we do not make one commun function with the save side
   // because on the save side we may need to create the .gdl/ PATH ...
   int result, debug=0;
+#ifdef _WIN32
+  char *homeDir = getenv( "HOMEPATH");
+#else
   char *homeDir = getenv( "HOME");
+#endif
   if (homeDir != NULL)
   {
     string pathToGDL_history;
@@ -1610,7 +1693,11 @@ historyIntialized = true;
 
   // go into main loop
   for (;;) {
+#if defined (_MSC_VER) && _MSC_VER < 1800
+    _clearfp();
+#else
     feclearexcept(FE_ALL_EXCEPT);
+#endif
 
     try
       {
