@@ -19,34 +19,35 @@
 
 #include "includefirst.hpp"
 
-#include "graphics.hpp"
+#include "graphicsdevice.hpp"
 #include "image.hpp"
-
-// #define ToXColor(a) (((0xFF & (a)) << 8) | (a))
-
-// #define free_mem(a) \
-//     if (a != NULL) { free((void *) a); a = NULL; }
+#ifdef HAVE_LIBWXWIDGETS
+#include "gdlwxstream.hpp"
+#endif
 
 using namespace std;
 
 namespace lib {
 
 
-  void tv( EnvT* e)
-  {
-    Graphics* actDevice = Graphics::GetDevice();
-    actDevice->TV( e);
-  }
-
   BaseGDL* tvrd( EnvT* e)
   {
-    Graphics* actDevice = Graphics::GetDevice();
-    return actDevice->TVRD( e);
+      // when !d.name == Null  we do nothing !
+      GDLGStream* actStream = GraphicsDevice::GetDevice()->GetStream();
+      if (actStream == NULL) e->Throw("Unable to create window.");
+
+#ifdef HAVE_LIBWXWIDGETS
+      if (actStream->HasImage()) // Currently just for WXStream
+        return static_cast<GDLWXStream*>(actStream)->GetImage(e);
+      else
+#endif
+        return GraphicsDevice::GetDevice()->TVRD( e);
   }
+#define MAX_COLORS 256
 
   void loadct( EnvT* e) // = LOADCT_INTERNALGDL for exclusive use by LOADCT
   {
-    SizeT nCT = Graphics::N_CT();
+    SizeT nCT = GraphicsDevice::N_CT();
 
     static int get_namesIx = e->KeywordIx( "GET_NAMES"); 
     bool get_names = e->KeywordPresent( get_namesIx);
@@ -56,7 +57,7 @@ namespace lib {
 
       DStringGDL* names = new DStringGDL( nCT, BaseGDL::NOZERO);
       for( SizeT i=0; i<nCT; ++i)
-        (*names)[ i] = Graphics::GetCT( i)->Name();
+        (*names)[ i] = GraphicsDevice::GetCT( i)->Name();
 
       e->SetKW( get_namesIx, names);
       return; //correct behaviour.
@@ -66,24 +67,23 @@ namespace lib {
 
     DLong iCT;
 
-    DByte r[256], g[256], b[256];
-    PLINT rint[256], gint[256], bint[256];
+    DByte r[MAX_COLORS], g[MAX_COLORS], b[MAX_COLORS];
+    PLINT rint[MAX_COLORS], gint[MAX_COLORS], bint[MAX_COLORS];
     //load original table
-    GDLCT* actCT = Graphics::GetCT();
-    actCT->Get(rint,gint,bint,256);
+    GDLCT* actCT = GraphicsDevice::GetCT();
+    actCT->Get(rint,gint,bint,MAX_COLORS);
 
 
     e->AssureLongScalarPar( 0, iCT);
     if( iCT < 0 || iCT >= nCT)
       e->Throw( "Table number must be from 0 to "+i2s(nCT-1));
     
-    Graphics* actDevice = Graphics::GetDevice();
+    GraphicsDevice* actDevice = GraphicsDevice::GetDevice();
     GDLGStream* actStream = actDevice->GetStream( false); // no open
 
-    Graphics::LoadCT( iCT);
+    GraphicsDevice::LoadCT( iCT);
     //new table is:
-    actCT = Graphics::GetCT();
-#define MAX_COLORS 256
+    actCT = GraphicsDevice::GetCT();
     DLong bottom=0;
     DLong ncolors=MAX_COLORS;
     if ( e->KeywordSet ( "BOTTOM" ) ) e->AssureLongScalarKWIfPresent ( "BOTTOM", bottom );
@@ -93,7 +93,6 @@ namespace lib {
     if (ncolors < 1) ncolors=1;
     if (ncolors > MAX_COLORS) ncolors=MAX_COLORS;
     if (bottom+ncolors > MAX_COLORS) ncolors=MAX_COLORS-bottom;
-#undef MAX_COLORS
     for( SizeT i=0, j=bottom ; j<bottom+ncolors; ++i, ++j) {
       actCT->Get( i, r[ i], g[ i], b[ i]);
       //update section of colors
@@ -118,8 +117,9 @@ namespace lib {
     }
 
     if (actStream != NULL)
-      actStream->scmap1( rint, gint, bint, 256);
+      actStream->scmap0( rint, gint, bint, MAX_COLORS);
   }
+#undef MAX_COLORS
 
 } // namespace
 
