@@ -54,7 +54,11 @@
 #include "matrix_cholesky.hpp"
 #endif
 
+#include "matrix_invert.hpp"
+
+#if defined(USE_LIBPROJ4)||defined(USE_LIBPROJ4_NEW)
 #include "gshhs.hpp"
+#endif
 
 using namespace std;
 
@@ -63,31 +67,46 @@ void LibInit_jmg()
   const char KLISTEND[] = "";
 
   // the following by Joel Gales <jomoga@users.sourceforge.net>
-  const string sizeKey[]={"L64","DIMENSIONS","FILE_LUN","N_DIMENSIONS",
-			  "N_ELEMENTS","STRUCTURE","TNAME","TYPE",KLISTEND};
-  new DLibFunRetNew(lib::size,string("SIZE"),1,sizeKey);
-  new DLibFunRetNew(lib::fstat,string("FSTAT"),1);
+  const string sizeKey[]={"L64","DIMENSIONS","FILE_LUN","FILE_OFFSET",
+			  "N_DIMENSIONS","N_ELEMENTS",
+			  "SNAME","STRUCTURE","TNAME","TYPE",KLISTEND};
+  new DLibFunRetNew(lib::size_fun,string("SIZE"),1,sizeKey);
+
+  new DLibFunRetNew(lib::fstat_fun,string("FSTAT"),1);
+
   const string routine_namesKey[]={"LEVEL","VARIABLES","FETCH","ARG_NAME",
 				   "STORE","S_FUNCTIONS","S_PROCEDURES",
 				   KLISTEND};
   new DLibFun(lib::routine_names_value,string("ROUTINE_NAMES"),-1,routine_namesKey);
+  
+  // the 2 following by Alain C. and Levan L.
+  const string isaKey[]={"ARRAY","FILE","NULL","NUMBER","SCALAR",KLISTEND};
+  new DLibFunRetNew(lib::isa_fun,string("ISA"),2,isaKey);
 
-  const string invertKey[]={"DOUBLE",KLISTEND};
+  new DLibFunRetNew(lib::typename_fun,string("TYPENAME"),1);
 
+  // the following (Chol and Invert) by Alain C. and Nodar K.
+  // Since we have a difference between the IDL way and the Eigen way
+  // we (temporary) remove these 2 codes in the CVS of GDL
+  // Help welcome. 
 #if defined(USE_EIGEN)
-  //  new DLibFunRetNew(lib::invert_fun2,string("INVERT_EIG"),2,invertKey);
-  new DLibPro(lib::choldc_pro,string("CHOLDC"),3,invertKey);
-  new DLibFunRetNew(lib::cholsol_fun,string("CHOLSOL"),4,invertKey);
+  const string cholKey[]={"DOUBLE",KLISTEND};
+  new DLibPro(lib::choldc_pro,string("CHOLDC"),2,cholKey);
+  new DLibFunRetNew(lib::cholsol_fun,string("CHOLSOL"),3,cholKey);
 
-  const string lacholKey[]={"DOUBLE","STATUS","UPPER",KLISTEND};
-  new DLibPro(lib::la_choldc_pro,string("LA_CHOLDC"),4,lacholKey);
-  new DLibFunRetNew(lib::la_cholsol_fun,string("LA_CHOLSOL"),4,lacholKey);
+  //
+  const string lacholdcKey[]={"DOUBLE","STATUS","UPPER",KLISTEND};
+  new DLibPro(lib::la_choldc_pro,string("LA_CHOLDC"),1,lacholdcKey);
+  const string lacholsolKey[]={"DOUBLE","STATUS",KLISTEND};
+  new DLibFunRetNew(lib::la_cholsol_fun,string("LA_CHOLSOL"),2,lacholsolKey);
 #endif
 
 #if defined(HAVE_LIBGSL) && defined(HAVE_LIBGSLCBLAS)
   
-  new DLibFunRetNew(lib::invert_fun,string("INVERT"),2,invertKey);
+  const string invertKey[]={"DOUBLE","GSL","EIGEN",KLISTEND};
+  new DLibFunRetNew(lib::AC_invert_fun,string("INVERT"),2,invertKey);
 
+  // if FFTw not available, FFT in the GSL used (slower)
   const string fftKey[]={"DOUBLE","INVERSE","OVERWRITE","DIMENSION",KLISTEND};
 #if defined(USE_FFTW)
   new DLibFun(lib::fftw_fun,string("FFT"),2,fftKey);
@@ -121,27 +140,30 @@ void LibInit_jmg()
   const string rk4Key[]={"DOUBLE",KLISTEND};
   new DLibFunRetNew(lib::rk4jmg_fun,string("RK4JMG"),5,rk4Key);
 
-#if defined(USE_LIBPROJ4)
-  const string map_proj_forwardKey[]={"CONNECTIVITY","FILL",
-				      "MAP_STRUCTURE","POLYGONS",
-				      "POLYLINES","RADIANS",KLISTEND};
+
+#if defined(USE_LIBPROJ4)||defined(USE_LIBPROJ4_NEW)
+  const string map_proj_forwardKey[]={"MAP_STRUCTURE","RADIANS","POLYGONS","POLYLINES","CONNECTIVITY","FILL",KLISTEND};
   new DLibFunRetNew(lib::map_proj_forward_fun,
-	      string("MAP_PROJ_FORWARD"),3,map_proj_forwardKey);
+	      string("MAP_PROJ_FORWARD"),2,map_proj_forwardKey,NULL);
 
-  const string map_proj_inverseKey[]={"RADIANS",KLISTEND};
+  const string map_proj_inverseKey[]={"RADIANS","MAP_STRUCTURE",KLISTEND};
   new DLibFunRetNew(lib::map_proj_inverse_fun,
-	      string("MAP_PROJ_INVERSE"),3,map_proj_inverseKey);
-
-#endif
+	      string("MAP_PROJ_INVERSE"),2,map_proj_inverseKey);
+//dummy functions for compatibility support of GCTP projections 
+  new DLibPro(lib::map_proj_gctp_forinit,string("MAP_PROJ_GCTP_FORINIT"),4);
+  new DLibPro(lib::map_proj_gctp_revinit,string("MAP_PROJ_GCTP_REVINIT"),4);
 
   // SA: GSHHS dataset
-  // TODO: COASTS, CONTINENTS, ORIENTATION, LIMIT,
-  //       MAP_STRUCTURE, MLINESTYLE, MLINETHICK, SPACING, T3D, ZVALUE
-  const string map_continentsKey[] = {"COLOR", "RIVERS", "COUNTRIES", 
-    "HIRES", "FILL_CONTINENTS", KLISTEND};
-  const string map_continentsWarnKey[] = {"USA", "COASTS", KLISTEND};
+  // TODO: USA, ORIENTATION, LIMIT,
+  //       , MLINESTYLE, MLINETHICK, SPACING, T3D, ZVALUE
+  const string map_continentsKey[] = {"COLOR", "RIVERS", "COUNTRIES", "COASTS", "CONTINENTS", "USA", 
+    "HIRES", "FILL_CONTINENTS", "MAP_STRUCTURE",KLISTEND};
+  const string map_continentsWarnKey[] = {"ORIENTATION", "LIMIT",
+         "MLINESTYLE", "MLINETHICK", "SPACING", "T3D", "ZVALUE", KLISTEND};
   new DLibPro(lib::map_continents, string("MAP_CONTINENTS"), 0, 
     map_continentsKey, map_continentsWarnKey);
+#endif
+  
 
 
   const string convert_coordKey[]={"DATA","DEVICE","NORMAL","T3D","DOUBLE",
@@ -281,8 +303,9 @@ void LibInit_jmg()
  
 #endif
 
-  const string tvKey[]={"TRUE","NORMAL","CHANNEL","XSIZE","YSIZE","ORDER","DEVICE","DATA",KLISTEND};
-  new DLibPro(lib::tv,string("TV"),4,tvKey);
+  const string tvKey[]={"TRUE","NORMAL","CHANNEL","XSIZE","YSIZE","ORDER","DEVICE","DATA","T3D","Z","CENTIMETERS","INCHES",KLISTEND};
+  const string tvWarnKey[]={"WORDS",KLISTEND};
+  new DLibPro(lib::tv_image,string("TV"),4,tvKey,tvWarnKey);
 
   const string tvrdKey[]={"CHANNEL","ORDER","TRUE","WORDS",KLISTEND};
   new DLibFunRetNew(lib::tvrd,string("TVRD"),5,tvrdKey);
@@ -291,46 +314,6 @@ void LibInit_jmg()
   const string loadctWarnKey[]={"FILE",KLISTEND};
   new DLibPro(lib::loadct,string("LOADCT_INTERNALGDL"),1,loadctKey,loadctWarnKey);
 
-  const string widget_baseKey[] = {"ALIGN_BOTTOM","ALIGN_CENTER","ALIGN_LEFT","ALIGN_RIGHT","ALIGN_TOP","MBAR","MODAL","BASE_ALIGN_BOTTOM","BASE_ALIGN_CENTER","BASE_ALIGN_LEFT","BASE_ALIGN_RIGHT","BASE_ALIGN_TOP","COLUMN","ROW","CONTEXT_EVENTS","CONTEXT_MENU","EVENT_FUNC","EVENT_PRO","EXCLUSIVE","NONEXCLUSIVE","FLOATING","FRAME","FUNC_GET_VALUE","GRID_LAYOUT","GROUP_LEADER","KBRD_FOCUS_EVENTS","KILL_NOTIFY","MAP","NO_COPY","NOTIFY_REALIZE","PRO_SET_VALUE","SCR_XSIZE","SCR_YSIZE","SCROLL","SENSITIVE","SPACE","TITLE","TLB_FRAME_ATTR","TLB_ICONIFY_EVENTS","TLB_KILL_REQUEST_EVENTS","TLB_MOVE_EVENTS","TLB_SIZE_EVENTS","TOOLBAR","TRACKING_EVENTS","UNITS","UNAME","UVALUE","XOFFSET","XPAD","XSIZE","X_SCROLL_SIZE","YOFFSET","YPAD","YSIZE","Y_SCROLL_SIZE","DISPLAY_NAME","RESOURCE_NAME","RNAME_MBAR",KLISTEND};
-  new DLibFunRetNew(lib::widget_base,string("WIDGET_BASE"),1,widget_baseKey);
-
-  const string widget_buttonKey[] = {"ACCELERATOR","ALIGN_CENTER","ALIGN_LEFT","ALIGN_RIGHT","BITMAP","CHECKED_MENU","DYNAMIC_RESIZE","EVENT_FUNC","EVENT_PRO","FONT","FRAME","FUNC_GET_VALUE","GROUP_LEADER","HELP","KILL_NOTIFY","MENU","NO_COPY","NO_RELEASE","NOTIFY_REALIZE","PRO_SET_VALUE","PUSHBUTTON_EVENTS","SCR_XSIZE","SCR_YSIZE","SENSITIVE","SEPARATOR","TAB_MODE","TOOLTIP","TRACKING_EVENTS","UNAME","UNITS","UVALUE","VALUE","X_BITMAP_EXTRA","XOFFSET","XSIZE","YOFFSET","YSIZE",KLISTEND};
-  new DLibFunRetNew(lib::widget_button,string("WIDGET_BUTTON"),1,widget_buttonKey);
-
-  const string widget_droplistKey[] = {"DYNAMIC_RESIZE","EVENT_FUNC","EVENT_PRO","FONT","FRAME","FUNC_GET_VALUE","GROUP_LEADER","KILL_NOTIFY","NO_COPY","NOTIFY_REALIZE","PRO_SET_VALUE","RESOURCE_NAME","SCR_XSIZE","SCR_YSIZE","SENSITIVE","TAB_MODE","TITLE","TRACKING_EVENTS","UNAME","UNITS","UVALUE","VALUE","XOFFSET","XSIZE","YOFFSET","YSIZE",KLISTEND};
-  new DLibFunRetNew(lib::widget_droplist,string("WIDGET_DROPLIST"),1,widget_droplistKey);
-
-	const string widget_listKey[] = {"CONTEXT_EVENTS","EVENT_FUNC","EVENT_PRO","FONT","FRAME","FUNC_GET_VALUE","GROUP_LEADER","KILL_NOTIFY","MULTIPLE","NO_COPY","NOTIFY_REALIZE","PRO_SET_VALUE","RESOURCE_NAME","SCR_XSIZE","SCR_YSIZE","SENSITIVE","TAB_MODE","TRACKING_EVENTS","UNAME","UNITS","UVALUE","VALUE","XOFFSET","XSIZE","YOFFSET","YSIZE",KLISTEND};
-	new DLibFunRetNew(lib::widget_list,string("WIDGET_LIST"),1,widget_listKey);
-
-	const string widget_bgroupKey[] =
-	{"BUTTON_UVALUE","COLUMN","EVENT_FUNC","EXCLUSIVE","NONEXCLUSIVE","SPACE","XPAD","YPAD","FONT","FRAME","IDS","LABEL_LEFT","LABEL_TOP","MAP","NO_RELEASE","RETURN_ID","RETURN_INDEX","RETURN_NAME","ROW","SCROLL","SET_VALUE","TAB_MODE","X_SCROLL_SIZE","Y_SCROLL_SIZE","SET_VALUE","UNAME","UVALUE","XOFFSET","XSIZE","YOFFSET","YSIZE",KLISTEND};
-	new DLibFunRetNew(lib::widget_bgroup,string("CW_BGROUP"),2,widget_bgroupKey);
-
-  const string widget_textKey[] = {"ALL_EVENTS","CONTEXT_EVENTS","EDITABLE","EVENT_FUNC","EVENT_PRO","FONT","FRAME","FUNC_GET_VALUE","GROUP_LEADER","IGNORE_ACCELERATORS","KBRD_FOCUS_EVENTS","KILL_NOTIFY","NO_COPY","NO_NEWLINE","NOTIFY_REALIZE","PRO_SET_VALUE","RESOURCE_NAME","SCR_XSIZE","SCR_YSIZE","SCROLL","SENSITIVE","TAB_MODE","TRACKING_EVENTS","UNAME","UNITS","UVALUE","VALUE","WRAP","XOFFSET","XSIZE","YOFFSET","YSIZE",KLISTEND};
-  new DLibFunRetNew(lib::widget_text,string("WIDGET_TEXT"),1,widget_textKey);
-
-  const string widget_labelKey[] = {"ALL_EVENTS","CONTEXT_EVENTS","EDITABLE","EVENT_FUNC","EVENT_PRO","FONT","FRAME","FUNC_GET_VALUE","GROUP_LEADER","IGNORE_ACCELERATORS","KBRD_FOCUS_EVENTS","KILL_NOTIFY","NO_COPY","NO_NEWLINE","NOTIFY_REALIZE","PRO_SET_VALUE","RESOURCE_NAME","SCR_XSIZE","SCR_YSIZE","SCROLL","SENSITIVE","TAB_MODE","TRACKING_EVENTS","UNAME","UNITS","UVALUE","VALUE","WRAP","XOFFSET","XSIZE","YOFFSET","YSIZE",KLISTEND};
-  new DLibFunRetNew(lib::widget_label,string("WIDGET_LABEL"),1,widget_labelKey);
-
-  const string widget_infoKey[] = {"VALID","MODAL","MANAGED","XMANAGER_BLOCK","CHILD","VERSION",
-				   KLISTEND};
-  new DLibFunRetNew(lib::widget_info,string("WIDGET_INFO"),1,widget_infoKey);
-
-  const string widget_eventKey[] = {"XMANAGER_BLOCK","DESTROY",
-				    KLISTEND};
-  new DLibFunRetNew(lib::widget_event,string("WIDGET_EVENT"),1,widget_eventKey);
-
-  const string widget_controlKey[] = {"REALIZE","MANAGED","EVENT_PRO",
-				      "XMANAGER_ACTIVE_COMMAND","DESTROY",
-				      "GET_UVALUE","SET_UVALUE","SET_VALUE",
-				      "MAP","FUNC_GET_VALUE","PRO_SET_VALUE",
-				      "SET_UNAME","NO_COPY","SET_BUTTON",
-				      "SET_DROPLIST_SELECT","SENSITIVE",
-				      "GET_VALUE",KLISTEND};
-  new DLibPro(lib::widget_control,string("WIDGET_CONTROL"),1, 
-	      widget_controlKey);
-
   // call_external (by Christoph Fuchs)
   const string call_externalKey[] = {"VALUE", "ALL_VALUE", "RETURN_TYPE",
                "B_VALUE", "I_VALUE", "L_VALUE", "F_VALUE", "D_VALUE",
@@ -338,4 +321,149 @@ void LibInit_jmg()
                "UNLOAD", "ALL_GDL", "STRUCT_ALIGN_BYTES", KLISTEND };
   new DLibFunRetNew(lib::call_external, string("CALL_EXTERNAL"), -1, call_externalKey);
 
+// WIDGET_ functions
+#define WIDGET_COMMON_KEYWORDS "EVENT_FUNC", "EVENT_PRO", "FUNC_GET_VALUE", "KILL_NOTIFY" \
+    , "NOTIFY_REALIZE" , "PRO_SET_VALUE" , "NO_COPY" , "GROUP_LEADER"\
+    , "SCR_XSIZE" , "SCR_YSIZE" , "SCROLL" , "SENSITIVE" \
+    , "UNAME" , "UNITS" , "UVALUE" , "XOFFSET" \
+    , "XSIZE" , "YOFFSET" , "YSIZE" , "FRAME"
+
+  const string widget_drawKey[] = { WIDGET_COMMON_KEYWORDS
+    , "APP_SCROLL"
+    , "BUTTON_EVENTS"
+    , "CLASSNAME"
+    , "COLOR_MODEL"
+    , "COLORS"
+    , "DRAG_NOTIFY"
+    , "DROP_EVENTS"
+    , "EXPOSE_EVENTS"
+    , "GRAPHICS_LEVEL"
+    , "IGNORE_ACCELERATORS"
+    , "KEYBOARD_EVENTS"
+    , "MOTION_EVENTS"
+    , "RENDERER"
+    , "RESOURCE_NAME"
+    , "RETAIN"
+    , "TOOLTIP"
+    , "TRACKING_EVENTS"
+    , "VIEWPORT_EVENTS"
+    , "WHEEL_EVENTS"
+    , "X_SCROLL_SIZE"
+    , "Y_SCROLL_SIZE"
+    ,KLISTEND};
+  new DLibFunRetNew(lib::widget_draw,string("WIDGET_DRAW"),1,widget_drawKey);
+  
+  const string widget_tableKey[] = {WIDGET_COMMON_KEYWORDS
+  , "ALIGNMENT"
+  , "ALL_EVENTS"
+  , "AM_PM"
+  , "BACKGROUND_COLOR"
+  , "COLUMN_LABELS"
+  , "COLUMN_MAJOR"
+  , "ROW_MAJOR"
+  , "COLUMN_WIDTHS"
+  , "CONTEXT_EVENTS"
+  , "DAYS_OF_WEEK"
+  , "DISJOINT_SELECTION"
+  , "EDITABLE"
+  , "FONT"
+  , "FOREGROUND_COLOR"
+  , "FORMAT"
+  , "GROUP_LEADER"
+  , "IGNORE_ACCELERATORS"
+  , "KBRD_FOCUS_EVENTS"
+  , "MONTHS"
+  , "NO_COLUMN_HEADERS"
+  , "NO_HEADERS"
+  , "NO_ROW_HEADERS"
+  , "RESIZEABLE_COLUMNS"
+  , "RESIZEABLE_ROWS"
+  , "ROW_HEIGHTS"
+  , "ROW_LABELS"
+  , "TAB_MODE"
+  , "TRACKING_EVENTS"
+  , "VALUE"
+  , "X_SCROLL_SIZE"
+  , "Y_SCROLL_SIZE"
+  , KLISTEND};
+  new DLibFunRetNew(lib::widget_table,string("WIDGET_TABLE"),1,widget_tableKey);
+  
+  const string widget_treeKey[] = {WIDGET_COMMON_KEYWORDS
+  , "ALIGN_BOTTOM"
+  , "ALIGN_CENTER"
+  , "ALIGN_LEFT"
+  , "ALIGN_RIGHT"
+  , "ALIGN_TOP"
+  , "BITMAP"
+  , "CHECKBOX"
+  , "CHECKED"
+  , "DRAG_NOTIFY"
+  , "DRAGGABLE"
+  , "EXPANDED"
+  , "FOLDER"
+  , "GROUP_LEADER"
+  , "INDEX"
+  , "MASK"
+  , "MULTIPLE"
+  , "NO_BITMAPS"
+  , "TAB_MODE"
+  , "TOOLTIP"
+  , KLISTEND};
+//   , "CONTEXT_EVENTS"
+//   , "TRACKING_EVENTS"
+//   , "DROP_EVENTS"
+  new DLibFunRetNew(lib::widget_base,string("WIDGET_TREE"),1,widget_treeKey);
+  
+  const string widget_baseKey[] = {WIDGET_COMMON_KEYWORDS,"ALIGN_BOTTOM","ALIGN_CENTER","ALIGN_LEFT","ALIGN_RIGHT","ALIGN_TOP","MBAR","MODAL","BASE_ALIGN_BOTTOM","BASE_ALIGN_CENTER","BASE_ALIGN_LEFT","BASE_ALIGN_RIGHT","BASE_ALIGN_TOP","COLUMN","ROW","CONTEXT_EVENTS","CONTEXT_MENU","EXCLUSIVE","NONEXCLUSIVE","FLOATING","GRID_LAYOUT","KBRD_FOCUS_EVENTS","MAP","SPACE","TITLE","TLB_FRAME_ATTR","TLB_ICONIFY_EVENTS","TLB_KILL_REQUEST_EVENTS","TLB_MOVE_EVENTS","TLB_SIZE_EVENTS","TOOLBAR","TRACKING_EVENTS","XPAD","X_SCROLL_SIZE","YPAD","Y_SCROLL_SIZE","DISPLAY_NAME","RESOURCE_NAME","RNAME_MBAR","TAB_MODE",KLISTEND};
+  new DLibFunRetNew(lib::widget_base,string("WIDGET_BASE"),1,widget_baseKey);
+
+  const string widget_buttonKey[] = {WIDGET_COMMON_KEYWORDS,"ACCELERATOR","ALIGN_CENTER","ALIGN_LEFT","ALIGN_RIGHT","BITMAP","CHECKED_MENU","DYNAMIC_RESIZE","FONT","HELP","MENU","NO_COPY","NO_RELEASE","PUSHBUTTON_EVENTS","SEPARATOR","TAB_MODE","TOOLTIP","TRACKING_EVENTS","VALUE","X_BITMAP_EXTRA",KLISTEND};
+  new DLibFunRetNew(lib::widget_button,string("WIDGET_BUTTON"),1,widget_buttonKey);
+
+  const string widget_droplistKey[] = {WIDGET_COMMON_KEYWORDS,"DYNAMIC_RESIZE","FONT","RESOURCE_NAME","TAB_MODE","TITLE","TRACKING_EVENTS","VALUE",KLISTEND};
+  new DLibFunRetNew(lib::widget_droplist,string("WIDGET_DROPLIST"),1,widget_droplistKey);
+
+  const string widget_comboboxKey[] = {WIDGET_COMMON_KEYWORDS,"EDITABLE","DYNAMIC_RESIZE","FONT","RESOURCE_NAME","TAB_MODE","TITLE","TRACKING_EVENTS","VALUE",KLISTEND};
+  new DLibFunRetNew(lib::widget_combobox,string("WIDGET_COMBOBOX"),1,widget_comboboxKey);
+
+  const string widget_listKey[] = {WIDGET_COMMON_KEYWORDS,"CONTEXT_EVENTS","FONT","MULTIPLE","RESOURCE_NAME","TAB_MODE","TRACKING_EVENTS","VALUE",KLISTEND};
+  new DLibFunRetNew(lib::widget_list,string("WIDGET_LIST"),1,widget_listKey);
+
+// 	const string widget_bgroupKey[] =
+// 	{"BUTTON_UVALUE","COLUMN","EVENT_FUNC","EXCLUSIVE","NONEXCLUSIVE","SPACE","XPAD","YPAD","FONT","FRAME","IDS","LABEL_LEFT","LABEL_TOP","MAP","NO_RELEASE","RETURN_ID","RETURN_INDEX","RETURN_NAME","ROW","SCROLL","SET_VALUE","TAB_MODE","X_SCROLL_SIZE","Y_SCROLL_SIZE","SET_VALUE","UNAME","UVALUE","XOFFSET","XSIZE","YOFFSET","YSIZE",KLISTEND};
+// 	new DLibFunRetNew(lib::widget_bgroup,string("CW_BGROUP"),2,widget_bgroupKey);
+
+  const string widget_textKey[] = {WIDGET_COMMON_KEYWORDS,"ALL_EVENTS","CONTEXT_EVENTS","EDITABLE","FONT","IGNORE_ACCELERATORS","KBRD_FOCUS_EVENTS","NO_NEWLINE","RESOURCE_NAME","TAB_MODE","TRACKING_EVENTS","VALUE","WRAP",KLISTEND};
+  new DLibFunRetNew(lib::widget_text,string("WIDGET_TEXT"),1,widget_textKey);
+
+  const string widget_labelKey[] = {WIDGET_COMMON_KEYWORDS,"ALL_EVENTS","CONTEXT_EVENTS","EDITABLE","FONT","IGNORE_ACCELERATORS","KBRD_FOCUS_EVENTS","NO_NEWLINE","RESOURCE_NAME","TAB_MODE","TRACKING_EVENTS","VALUE","WRAP",KLISTEND};
+  const string widget_labelWarnKey[] = {"ALIGN_CENTER","ALIGN_LEFT","ALIGN_RIGHT", KLISTEND};
+  new DLibFunRetNew(lib::widget_label,string("WIDGET_LABEL"),1,widget_labelKey,widget_labelWarnKey);
+
+  const string widget_tabKey[] = {WIDGET_COMMON_KEYWORDS,"ALIGN_BOTTOM","ALIGN_CENTER","ALIGN_LEFT","ALIGN_RIGHT","ALIGN_TOP","MULTILINE","LOCATION","TAB_MODE","TRACKING_EVENTS", KLISTEND};
+  new DLibFunRetNew(lib::widget_tab,string("WIDGET_TAB"),1,widget_tabKey);
+
+  const string widget_slider[] = {WIDGET_COMMON_KEYWORDS,"TITLE"/*TODO*/,"DRAG","VALUE","MINIMUM","MAXIMUM","VERTICAL","SUPPRESS_VALUE", KLISTEND};
+  new DLibFunRetNew(lib::widget_slider,string("WIDGET_SLIDER"),1,widget_slider);
+
+  
+  const string widget_infoKey[] = {"VALID_ID","MODAL","MANAGED","XMANAGER_BLOCK","CHILD","VERSION","GEOMETRY","UNAME",
+				   KLISTEND};
+  new DLibFunRetNew(lib::widget_info,string("WIDGET_INFO"),1,widget_infoKey);
+
+  const string widget_eventKey[] = {"XMANAGER_BLOCK","DESTROY",
+				    KLISTEND};
+  new DLibFunRetNew(lib::widget_event,string("WIDGET_EVENT"),1,widget_eventKey);
+
+  const string widget_ControlKey[] = {"REALIZE","MANAGED","EVENT_FUNC","EVENT_PRO",
+				      "XMANAGER_ACTIVE_COMMAND","DESTROY",
+				      "GET_UVALUE","SET_UVALUE","SET_VALUE",
+				      "MAP","FUNC_GET_VALUE","PRO_SET_VALUE",
+				      "SET_UNAME","NO_COPY","SET_BUTTON",
+				      "SET_DROPLIST_SELECT","SENSITIVE",
+				      "GET_VALUE","NO_NEWLINE",KLISTEND};
+  const string widget_WarnControlKey[] ={"DEFAULT_FONT","CLEAR_EVENTS","XOFFSET","YOFFSET","TLB_SET_TITLE","KILL_NOTIFY",KLISTEND};
+  new DLibPro(lib::widget_control,string("WIDGET_CONTROL"),1, 
+	      widget_ControlKey,widget_WarnControlKey);
+
 }
