@@ -1,5 +1,5 @@
 /***************************************************************************
-                          basegdl.cpp  -  base class for GDL datatypes
+                          nullgdl.cpp  -  class for GDL !NULL
                              -------------------
     begin                : July 22 2002
     copyright            : (C) 2002 by Marc Schellens
@@ -29,8 +29,11 @@ NullGDL* NullGDL::instance = NULL;
 
 NullGDL::~NullGDL()
 {
-  //throw GDLException("Attempt to destroy !NULL."); 
-  cerr << "Internal error: !NULL destructor called (GDL session still ok).\nPlease report at http://sourceforge.net/tracker/?group_id=97659&atid=618683" << endl;
+// it would be the fastest way, to just ignore it.
+// but ~BaseGDL() is called as well. And this must not be
+  cerr << "Internal error: !NULL destructor called.\n"
+	  "Save your work and restart GDL (GDL is still functional, but !NULL will not work anymore).\n"
+	  "Please report at http://sourceforge.net/tracker/?group_id=97659&atid=618683" << endl;
 }
 
 bool NullGDL::IsAssoc() const { return false;}
@@ -184,7 +187,7 @@ BaseGDL* NullGDL::Convert2( DType destTy, Convert2Mode mode)
 
 BaseGDL* NullGDL::GetTag() const 
 {
-  throw GDLException("NullGDL::GetTag(...) called.");
+  throw GDLException("!NULL not allowed in tag definition.");
 }
 
 BaseGDL* NullGDL::GetInstance() const
@@ -566,7 +569,9 @@ SizeT NullGDL::IFmtI( std::istream* is, SizeT offs, SizeT num, int width,
 {throw GDLException("NullGDL::IFmtI(...) called.");}
 
 BaseGDL* NullGDL::Convol( BaseGDL* kIn, BaseGDL* scaleIn, BaseGDL* bias,
-		 bool center, bool normalize, int edgeMode)
+		          bool center, bool normalize, int edgeMode,
+                          bool doNan, BaseGDL* missing, bool doMissing,
+                          BaseGDL* invalid, bool doInvalid)
 {
   throw GDLException("NullGDL::Convol(...) called.");
 }
