@@ -167,7 +167,9 @@ class NullGDL: public BaseGDL
 
   // library functions
   /*virtual*/ BaseGDL* Convol( BaseGDL* kIn, BaseGDL* scaleIn, BaseGDL* bias,
-			   bool center, bool normalize, int edgeMode);
+			       bool center, bool normalize, int edgeMode,
+                               bool doNan, BaseGDL* missing, bool doMissing,
+                               BaseGDL* invalid, bool doInvalid);
   /*virtual*/ BaseGDL* Rebin( const dimension& newDim, bool sample);
   // for STRUCT_ASSIGN
   /*virtual*/ void Assign( BaseGDL* src, SizeT nEl);
