@@ -26,7 +26,7 @@
 #include "dnodefactory.hpp"
 
 #include "objects.hpp"
-#include "graphics.hpp"
+#include "graphicsdevice.hpp"
 #include "preferences.hpp"
 #include "overload.hpp"
 
@@ -78,9 +78,9 @@ namespace structDesc {
 
 // for OpenMP
 DLong CpuTPOOL_NTHREADS;
-DLong CpuTPOOL_MIN_ELTS;
-DLong CpuTPOOL_MAX_ELTS;
-const DLong CpuTPOOL_MAX_ELTS_max = numeric_limits<DLong>::max();
+DLong64 CpuTPOOL_MIN_ELTS;
+DLong64 CpuTPOOL_MAX_ELTS;
+const DLong64 CpuTPOOL_MAX_ELTS_max = numeric_limits<DLong64>::max();
 
 // instantiate own AST factory
 //_DNodeFactory DNodeFactory;
@@ -88,7 +88,7 @@ antlr::ASTFactory DNodeFactory("DNode",DNode::factory);
 
 void ResetObjects()
 {
-  Graphics::DestroyDevices();
+  GraphicsDevice::DestroyDevices();
 
   fileUnits.clear();
   cerr << flush; cout << flush; clog << flush;
@@ -121,6 +121,7 @@ void InitStructs()
   SpDFloat  aFloat;
   SpDDouble aDouble;
   SpDLong   aLongArr8( dimension(8));
+  SpDLong64   aLong64Arr8( dimension(8));
   SpDPtr    aPtrRef;
   SpDObj    aObjRef;
 
@@ -178,21 +179,30 @@ void InitStructs()
   structDesc::GDL_HASHTABLEENTRY = gdlHashTE;
   
   // OBJECTS END =======================================================
-
-  
-  
-  DStructDesc* gdl_size = new DStructDesc( "IDL_SIZE");
-  gdl_size->AddTag("TYPE_NAME", &aString);
-  gdl_size->AddTag("STRUCTURE_NAME", &aString);
-  gdl_size->AddTag("TYPE", &aInt);
-  gdl_size->AddTag("FILE_LUN", &aInt);
-  gdl_size->AddTag("FILE_OFFSET",  &aLong);
-  gdl_size->AddTag("N_ELEMENTS",  &aLong);
-  gdl_size->AddTag("N_DIMENSIONS",  &aLong);
-  gdl_size->AddTag("DIMENSIONS",  &aLongArr8);
-  // insert into structList
-  structList.push_back(gdl_size);
-
+   
+  for (int big = 1; big >= 0; --big) 
+  {
+    DStructDesc* gdl_size = new DStructDesc( big ? "IDL_SIZE64" : "IDL_SIZE");
+    gdl_size->AddTag("TYPE_NAME", &aString);
+    gdl_size->AddTag("STRUCTURE_NAME", &aString);
+    gdl_size->AddTag("TYPE", &aInt);
+    gdl_size->AddTag("FILE_LUN", &aInt);
+    if (big) {
+      gdl_size->AddTag("FILE_OFFSET",  &aLong64);
+      gdl_size->AddTag("N_ELEMENTS",  &aLong64);
+    } else {
+      gdl_size->AddTag("FILE_OFFSET",  &aLong);
+      gdl_size->AddTag("N_ELEMENTS",  &aLong);
+    }
+    gdl_size->AddTag("N_DIMENSIONS",  &aLong);
+    if (big) {
+      gdl_size->AddTag("DIMENSIONS",  &aLong64Arr8);
+    } else {
+      gdl_size->AddTag("DIMENSIONS",  &aLongArr8);
+    }
+    // insert into structList
+    structList.push_back(gdl_size);
+  }
 
   for (int big = 1; big >= 0; --big) 
   {
@@ -303,6 +313,17 @@ void InitStructs()
   // insert into structList
   structList.push_back( dmachar);
 
+  // for internal usage
+  // attention: $WIDGET_MESSAGE would identify this as an unnamed struct
+  // see DStructDesc constructor
+  DStructDesc* widgmsg = new DStructDesc( "*WIDGET_MESSAGE*");
+  widgmsg->AddTag("ID", &aLong);
+  widgmsg->AddTag("TOP", &aLong);
+  widgmsg->AddTag("HANDLER", &aLong);
+  widgmsg->AddTag("MESSAGE", &aLong);
+  // insert into structList
+  structList.push_back( widgmsg);
+
   DStructDesc* widgbut = new DStructDesc( "WIDGET_BUTTON");
   widgbut->AddTag("ID", &aLong);
   widgbut->AddTag("TOP", &aLong);
@@ -315,18 +336,45 @@ void InitStructs()
   widgdlist->AddTag("ID", &aLong);
   widgdlist->AddTag("TOP", &aLong);
   widgdlist->AddTag("HANDLER", &aLong);
-  widgdlist->AddTag("SELECT", &aLong);
+  widgdlist->AddTag("INDEX", &aLong);
   // insert into structList
   structList.push_back( widgdlist);
 
+  DStructDesc* widgcbox = new DStructDesc( "WIDGET_COMBOBOX");
+  widgcbox->AddTag("ID", &aLong);
+  widgcbox->AddTag("TOP", &aLong);
+  widgcbox->AddTag("HANDLER", &aLong);
+  widgcbox->AddTag("INDEX", &aLong);
+  widgcbox->AddTag("STR", &aString);
+  // insert into structList
+  structList.push_back( widgcbox);
+
   DStructDesc* widglist = new DStructDesc( "WIDGET_LIST");
   widglist->AddTag("ID", &aLong);
   widglist->AddTag("TOP", &aLong);
   widglist->AddTag("HANDLER", &aLong);
-  widglist->AddTag("SELECT", &aLong);
+  widglist->AddTag("INDEX", &aLong);
+  widglist->AddTag("CLICKS", &aLong);
   // insert into structList
   structList.push_back( widglist);
 
+  DStructDesc* widgdtab = new DStructDesc( "WIDGET_TAB");
+  widgdtab->AddTag("ID", &aLong);
+  widgdtab->AddTag("TOP", &aLong);
+  widgdtab->AddTag("HANDLER", &aLong);
+  widgdtab->AddTag("TAB", &aLong);
+  // insert into structList
+  structList.push_back( widgdtab);
+
+  DStructDesc* widgdsl = new DStructDesc( "WIDGET_SLIDER");
+  widgdsl->AddTag("ID", &aLong);
+  widgdsl->AddTag("TOP", &aLong);
+  widgdsl->AddTag("HANDLER", &aLong);
+  widgdsl->AddTag("VALUE", &aLong);
+  widgdsl->AddTag("DRAG", &aInt);
+  // insert into structList
+  structList.push_back( widgdsl);
+
   DStructDesc* widgbgroup =  new DStructDesc( "WIDGET_BGROUP");
   widgbgroup->AddTag("ID", &aLong);
   widgbgroup->AddTag("TOP", &aLong);
@@ -336,13 +384,45 @@ void InitStructs()
   // insert into structList
   structList.push_back(widgbgroup);
 
-  DStructDesc* widgtxt = new DStructDesc( "WIDGET_TEXT");
-  widgtxt->AddTag("ID", &aLong);
-  widgtxt->AddTag("TOP", &aLong);
-  widgtxt->AddTag("HANDLER", &aLong);
-  widgtxt->AddTag("SELECT", &aLong);
+  DStructDesc* widgtxtc = new DStructDesc( "WIDGET_TEXT_CH");
+  widgtxtc->AddTag("ID", &aLong);
+  widgtxtc->AddTag("TOP", &aLong);
+  widgtxtc->AddTag("HANDLER", &aLong);
+  widgtxtc->AddTag("TYPE", &aInt); // 0
+  widgtxtc->AddTag("OFFSET", &aLong);
+  widgtxtc->AddTag("CH", &aByte);
   // insert into structList
-  structList.push_back( widgtxt);
+  structList.push_back( widgtxtc);
+
+  DStructDesc* widgtxtst = new DStructDesc( "WIDGET_TEXT_STR");
+  widgtxtst->AddTag("ID", &aLong);
+  widgtxtst->AddTag("TOP", &aLong);
+  widgtxtst->AddTag("HANDLER", &aLong);
+  widgtxtst->AddTag("TYPE", &aInt); // 1
+  widgtxtst->AddTag("OFFSET", &aLong);
+  widgtxtst->AddTag("STR", &aString);
+  // insert into structList
+  structList.push_back( widgtxtst);
+
+  DStructDesc* widgtxts = new DStructDesc( "WIDGET_TEXT_SEL");
+  widgtxts->AddTag("ID", &aLong);
+  widgtxts->AddTag("TOP", &aLong);
+  widgtxts->AddTag("HANDLER", &aLong);
+  widgtxts->AddTag("TYPE", &aInt); // 3
+  widgtxts->AddTag("OFFSET", &aLong);
+  widgtxts->AddTag("LENGTH", &aLong);
+  // insert into structList
+  structList.push_back( widgtxts);
+  
+  DStructDesc* widgtxtd = new DStructDesc( "WIDGET_TEXT_DEL");
+  widgtxtd->AddTag("ID", &aLong);
+  widgtxtd->AddTag("TOP", &aLong);
+  widgtxtd->AddTag("HANDLER", &aLong);
+  widgtxtd->AddTag("TYPE", &aInt); // 2
+  widgtxtd->AddTag("OFFSET", &aLong);
+  widgtxtd->AddTag("LENGTH", &aLong);
+  // insert into structList
+  structList.push_back( widgtxtd);
 
   DStructDesc* widgver = new DStructDesc( "WIDGET_VERSION");
   widgver->AddTag("STYLE", &aString);
@@ -350,6 +430,22 @@ void InitStructs()
   widgver->AddTag("RELEASE", &aString);
   // insert into structList
   structList.push_back( widgver);
+  
+  DStructDesc* widggeom = new DStructDesc( "WIDGET_GEOMETRY");
+  widggeom->AddTag("XOFFSET",&aFloat);
+  widggeom->AddTag("YOFFSET",&aFloat);
+  widggeom->AddTag("XSIZE",&aFloat);
+  widggeom->AddTag("YSIZE",&aFloat);
+  widggeom->AddTag("SCR_XSIZE",&aFloat);
+  widggeom->AddTag("SCR_YSIZE",&aFloat);
+  widggeom->AddTag("DRAW_XSIZE",&aFloat);
+  widggeom->AddTag("DRAW_YSIZE",&aFloat);
+  widggeom->AddTag("MARGIN",&aFloat);
+  widggeom->AddTag("XPAD",&aFloat);
+  widggeom->AddTag("YPAD",&aFloat);
+  widggeom->AddTag("SPACE",&aFloat);
+  // insert into structList
+  structList.push_back( widggeom);
 }
 
 void InitObjects()
@@ -370,12 +466,13 @@ void InitObjects()
   
   // graphic devices must be initialized after system variables
   // !D must already exist
-  Graphics::Init();
+  GraphicsDevice::Init();
 
   // preferences
   //  Preferences::Init();
 
 #ifdef HAVE_LIBWXWIDGETS
+
   // initialize widget system
   GDLWidget::Init();
 #endif
@@ -492,7 +589,8 @@ int get_suggested_omp_num_threads() {
   //    cout<<"OMP_NUM_THREADS is not defined"<<endl;
   
   //set number of threads for appropriate OS
-  int avload, nbofproc=omp_get_num_procs();
+  int avload = 0;
+  int nbofproc = omp_get_num_procs();
   FILE *iff;
     
 #if defined(__APPLE__) || defined(__MACH__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__bsdi__) || defined(__DragonFly__)
@@ -511,7 +609,7 @@ int get_suggested_omp_num_threads() {
       return default_num_threads;
     }
    
-#elif defined (__unix) || (__unix__)
+#elif defined (__unix) || defined(__unix__)
   iff=freopen("/proc/loadavg","r",stderr);
   fclose(stderr);
   if(!iff)
@@ -525,9 +623,10 @@ int get_suggested_omp_num_threads() {
       return default_num_threads;
     }
 
-#elif defined(_WIN32) || defined(__WIN32__) || defined(__WINDOWS__)
-  cout<<"is windows"<<endl;
-  iff= popen("wmic cpu get loadpercentage|more +1", "r");
+
+#elif defined(_WIN32)
+  //cout<<"get_suggested_omp_num_threads(): is windows"<<endl;
+  iff= _popen("wmic cpu get loadpercentage|more +1", "r");
   if (!iff)
     {
       return default_num_threads;
@@ -535,15 +634,18 @@ int get_suggested_omp_num_threads() {
   char buffer[4];
   char* c;
   c=fgets(buffer, sizeof(buffer), iff);
+  _pclose(iff);
   if(!c)
     {
       return default_num_threads;
     }
-  pclose(iff);
-  int cout=0;
-  while(buffer[count]!='\0' && buffer[count]!=' ')count++;
+  int count=0;
+  while(count < sizeof(buffer) && buffer[count]!='\0' && buffer[count]!=' ')count++;
   for(int i=1,j=1;i<=count;i++,j*=10)
-    avload+=(buffer[count-i]-'0')*j;
+  {
+    if( buffer[count-i] != ' ' && buffer[count-i] != '\t')
+      avload+=(buffer[count-i]-'0')*j;
+  }
   suggested_num_threads=nbofproc-(int)(avload*((float)nbofproc/100)+0.5);
   return suggested_num_threads;
 
@@ -555,18 +657,23 @@ int get_suggested_omp_num_threads() {
   //  cout << "Nb Procs.: " << nbofproc <<  endl;
   // cout << "nb Thead computed: " << nbofproc-(int)(avload+0.5) << endl;
 
+  // if the following is commented out, there is no return statement
+  // this lead to FILE_INFO in TEST_FILE_COPY fail
+#if !defined(_WIN32)
+  
   char buffer[4];
   char* c;
   c=fgets(buffer, sizeof(buffer), iff);
+  pclose(iff);
   if(!c)
     {
       return default_num_threads;
     }
-  pclose(iff);
   //   cout<<buffer[0]<<" "<<buffer[1]<<endl;
   avload=(buffer[0]-'0')+((buffer[2]-'0')>5?1:0);
 
   suggested_num_threads=nbofproc-avload;
+#endif  
   return suggested_num_threads;
 }
 #endif
