@@ -26,7 +26,9 @@ namespace lib {
 
   using namespace std;
 //  using std::isinf;
+#ifndef _MSC_VER
   using std::isnan;
+#endif
 
   class plot_call : public plotting_routine_call 
   {
@@ -149,11 +151,45 @@ private:
         }
       }
     }
-    // handle Log options
-    int xLogIx = e->KeywordIx("XLOG");
-    int yLogIx = e->KeywordIx("YLOG");
-    xLog = e->KeywordSet(xLogIx);
-    yLog = e->KeywordSet(yLogIx);
+
+    xLog=FALSE;
+    yLog=FALSE;
+
+    // handle Log options passing via Functions names PLOT_IO/OO/OI
+    // the behavior can be superseed by [xy]log or [xy]type
+    string ProName=e->GetProName();
+    if (ProName != "PLOT") {
+      if (ProName == "PLOT_IO") yLog=TRUE;
+      if (ProName == "PLOT_OI") xLog=TRUE;
+      if (ProName == "PLOT_OO") {
+	xLog=TRUE;
+	yLog=TRUE;
+      }
+    }
+
+    // handle Log options passing via Keywords
+    static int xTypeIx = e->KeywordIx("XTYPE");
+    static int yTypeIx = e->KeywordIx("YTYPE");
+    static int xLogIx = e->KeywordIx("XLOG");
+    static int yLogIx = e->KeywordIx("YLOG");
+
+    if (e->KeywordPresent(xLogIx)) xLog = e->KeywordSet(xLogIx);
+    if (e->KeywordPresent(yLogIx)) yLog = e->KeywordSet(yLogIx);
+
+    if (e->KeywordPresent(xTypeIx )) {
+      xLog=e->KeywordSet (xTypeIx );
+    } else {
+      xLog=e->KeywordSet (xLogIx );
+    }
+    
+    if (e->KeywordPresent(yTypeIx )) {
+      yLog=e->KeywordSet (yTypeIx );
+    } else {
+      yLog=e->KeywordSet (yLogIx);
+    }
+    
+    //cout << xLog << " " << yLog << endl;
+
     // compute adequate values for log scale, warn adequately...
     wasBadxLog = FALSE;
     wasBadyLog = FALSE;
@@ -310,7 +346,6 @@ private:
       if (gdlSet3DViewPortAndWorldCoordinates(e, actStream, plplot3d, xLog, yLog,
         xStart, xEnd, yStart, yEnd) == FALSE) return;
       gdlSetGraphicsForegroundColorFromKw(e, actStream);
-      gdlSetPlotCharthick(e, actStream);
 
       DDouble  t3xStart, t3xEnd, t3yStart, t3yEnd, t3zStart, t3zEnd;
       switch (axisExchangeCode) {
@@ -401,47 +436,27 @@ private:
 
       Data3d.zValue = zValue;
       Data3d.Matrix = plplot3d; //try to change for !P.T in future?
-        switch (axisExchangeCode) {
-          case NORMAL: //X->X Y->Y plane XY
             Data3d.x0=x0;
             Data3d.y0=y0;
             Data3d.xs=xs;
             Data3d.ys=ys;
+      switch (axisExchangeCode) {
+          case NORMAL: //X->X Y->Y plane XY
             Data3d.code = code012;
             break;
           case XY: // X->Y Y->X plane XY
-            Data3d.x0=0;
-            Data3d.y0=x0;
-            Data3d.xs=ys;
-            Data3d.ys=xs;
             Data3d.code = code102;
             break;
           case XZ: // Y->Y X->Z plane YZ
-            Data3d.x0=x0;
-            Data3d.y0=y0;
-            Data3d.xs=xs;
-            Data3d.ys=ys;
             Data3d.code = code210;
             break;
           case YZ: // X->X Y->Z plane XZ
-            Data3d.x0=x0;
-            Data3d.y0=y0;
-            Data3d.xs=xs;
-            Data3d.ys=ys;
             Data3d.code = code021;
             break;
           case XZXY: //X->Y Y->Z plane YZ
-            Data3d.x0=x0;
-            Data3d.y0=y0;
-            Data3d.xs=xs;
-            Data3d.ys=ys;
             Data3d.code = code120;
             break;
           case XZYZ: //X->Z Y->X plane XZ
-            Data3d.x0=x0;
-            Data3d.y0=y0;
-            Data3d.xs=xs;
-            Data3d.ys=ys;
             Data3d.code = code201;
             break;
         }
@@ -457,7 +472,6 @@ private:
 	      xStart, xEnd, yStart, yEnd, iso)==FALSE) return; //no good: should catch an exception to get out of this mess.
       //current pen color...
       gdlSetGraphicsForegroundColorFromKw(e, actStream);
-      gdlSetPlotCharthick(e, actStream);
 
       gdlBox(e, actStream, xStart, xEnd, yStart, yEnd, xLog, yLog);
     }
@@ -502,5 +516,9 @@ private:
     plot.call(e, 1);
   }
 
+  void plot_io(EnvT* e) {plot(e);}
+  void plot_oo(EnvT* e) {plot(e);}
+  void plot_oi(EnvT* e) {plot(e);}
+
 } // namespace
 
