@@ -31,7 +31,7 @@
 
 //#include "format.g"
 
-#ifdef _MSC_VER
+#if defined(_WIN32) && !defined(__CYGWIN__)
 #include <io.h>
 #define isatty _isatty
 #endif
@@ -144,6 +144,11 @@ namespace lib {
 	DString fmtString;
 	e->AssureScalarKW<DStringGDL>( 0, fmtString);
 
+	// removing "$" in input format : not used
+	std::size_t found =fmtString.find("$");
+	if (found!=std::string::npos)
+	  fmtString.erase(found,1); 
+
 	RefFMTNode fmtAST = GetFMTAST( fmtString);
 
 	FMTIn Formatter( fmtAST, is, e, parOffset, prompt);
@@ -271,7 +276,7 @@ namespace lib {
 	    else
 #endif
 	      {
-		posBeforeLast = is->tellg();
+// 		posBeforeLast = is->tellg();
 
 		parIn->FromStream( *is);
 
@@ -279,9 +284,16 @@ namespace lib {
 	      }
 	  }
       }
+      // read until end of line
       if( !lastParIsString && !is->eof()) // && is->peek() != '\n' && is->peek() != '\r')
       {
-	is->seekg( posBeforeLast);
+	streampos actPos = is->tellg();
+
+// 	is->seekg( posBeforeLast);
+	assert( actPos > 0);
+	
+	streamoff one = 1;
+	is->seekg( actPos-one);
 	
 	DStringGDL gdlString("");
 	gdlString.FromStream( *is);
@@ -328,6 +340,11 @@ namespace lib {
 	DString fmtString;
 	e->AssureScalarKW<DStringGDL>( 0, fmtString);
 
+	// removing "$" in input format : not used
+	std::size_t found =fmtString.find("$");
+	if (found!=std::string::npos)
+	  fmtString.erase(found,1); 
+
 	RefFMTNode fmtAST = GetFMTAST( fmtString);
 
 	FMTIn Formatter( fmtAST, &is, e, 1, NULL);
